from masonite.environment import env
from masonite.utils.location import base_path


DISKS = {
    "default": "local",
    "local": {"driver": "file", "path": base_path("storage/framework/filesystem")},
    "s3": {
        "driver": "s3",
        "client": env("AWS_CLIENT"),
        "secret": env("AWS_SECRET"),
        "bucket": env("AWS_BUCKET"),
    },
}

STATICFILES = {
    # folder          # template alias
    "storage/static": "static/",
    "storage/compiled": "assets/",
    "storage/public": "/",
    "vendor/filemanager": "filemanager/",
}



"""A WelcomeController Module."""
from masonite.views import View
from masonite.controllers import Controller


class WelcomeController(Controller):
    """WelcomeController Controller Class."""

    def show(self, view: View):
        return view.render("welcome")



import os
from pathlib import Path
import shutil
import time
from .filemanager_driver import FileManagerDriver


class LocalDriver(FileManagerDriver):
    def _get_path(self, extra=None) -> str:
        """Get the path to the filemanager directory"""

        request = self.application.make("request")
        folder = request.input("folder", None)

        path = self.root_path
        if folder:
            folders = folder.split(",")
            path = os.path.join(self.root_path, *folders)

        if extra:
            path = os.path.join(path, extra)

        return path

    def total_size(self):
        """Get the total size of the filemanager directory"""

        size = sum(file.stat().st_size for file in Path(self.root_path).rglob("*"))
        return self.convert_bytes(size)

    def exists(self, name) -> bool:
        """Check if a file/path exists in the filemanager directory"""

        return os.path.exists(self._get_path(name))

    def upload(self, file):
        """Upload a file to the filemanager directory"""

        request = self.application.make("request")
        folder = request.input("folder", None)
        path = "filemanager"

        if folder:
            path = "filemanager/{}".format(folder.replace(",", "/"))

        self.storage.disk("local").put_file(path, file)

    def create_folder(self, name) -> bool:
        """Create a folder in the filemanager directory"""

        try:
            path = self._get_path(name)
            if not self.exists(path):
                os.mkdir(self._get_path(name))
                return True
        except Exception as e:
            print(e)
        return False

    def rename(self, path, name) -> bool:
        """Rename a file/folder in the filemanager directory"""

        if os.path.exists(path):
            # check if path is file
            if os.path.isfile(path):
                name = "{name}.{ext}".format(name=name, ext=path.split(".")[-1])
            os.rename(path, self._get_path(name))
            return True
        return False

    def delete_folder(self, path) -> bool:
        """Delete a folder in the filemanager directory"""

        try:
            if os.path.exists(path):
                if len(os.listdir(path)) > 0:
                    shutil.rmtree(path)
                else:
                    os.rmdir(path)
                return True
        except Exception as e:
            print(e)
        return False

    def all_files(self):
        data = {
            "folders": [],
            "files": [],
            "total_size": self.total_size(),
        }

        path = self._get_path()

        with os.scandir(path) as entries:
            for item in entries:
                name = item.name
                byte_size = (
                    item.stat().st_size
                    if item.is_file()
                    else sum(file.stat().st_size for file in Path(item.path).rglob("*"))
                )
                size = self.convert_bytes(byte_size)
                created_at = time.ctime(item.stat().st_ctime)
                modified_at = time.ctime(item.stat().st_mtime)

                file_url = item.path.replace(self.root.get_path(""), "")
                file_url = "/uploads/" + file_url.replace("\\", "/")

                file_item = {
                    "name": name if item.is_dir() else item.name.split(".")[0],
                    "size": size,
                    "created": created_at,
                    "modified": modified_at,
                    "path": item.path,
                }

                if item.is_dir():
                    file_item["total_files"] = len(os.listdir(item.path))
                    data.get("folders").append(file_item)
                else:
                    file_item["url"] = file_url
                    file_item["mime"] = self._get_media_type(item)
                    data.get("files").append(file_item)

        return self.validate(data)



"""User Model."""
from masoniteorm.models import Model
from masoniteorm.scopes import SoftDeletesMixin
from masonite.authentication import Authenticates


class User(Model, SoftDeletesMixin, Authenticates):
    """User Model."""

    __fillable__ = ["name", "email", "password"]
    __hidden__ = ["password"]
    __auth__ = "email"



# from masonite.environment import env


STORES = {
    "default": "local",
    "local": {
        "driver": "file",
        "location": "storage/framework/cache"
        #
    },
    "redis": {
        "driver": "redis",
        "host": "127.0.0.1",
        "port": "6379",
        "password": "",
        "name": "project_name",
    },
    "memcache": {
        "driver": "memcache",
        "host": "127.0.0.1",
        "port": "11211",
        "password": "",
        "name": "project_name",
    },
}



from masonite.environment import env


BROADCASTS = {
    "default": "pusher",
    "pusher": {
        "driver": "pusher",
        "client": env("PUSHER_CLIENT"),
        "app_id": env("PUSHER_APP_ID"),
        "secret": env("PUSHER_SECRET"),
        "cluster": env("PUSHER_CLUSTER"),
        "ssl": False,
    },
}



import mimetypes
import os
from ..constants import NOT_IMPLEMENTED_MSG


class FileManagerDriver:
    def __init__(self) -> None:
        from wsgi import application

        self.application = application
        self.storage = application.make("storage")
        self.root = self.storage.disk("local")
        self.root_path = self.root.get_path("filemanager")

        if not os.path.exists(self.root_path):
            os.mkdir(self.root_path)

    def _get_path(self, extra=None) -> str:
        raise NotImplementedError(NOT_IMPLEMENTED_MSG)

    def _get_media_type(self, file) -> str:
        return mimetypes.guess_type(file.name)[0]

    def total_size(self) -> str:
        raise NotImplementedError(NOT_IMPLEMENTED_MSG)

    def upload(self, file):
        raise NotImplementedError(NOT_IMPLEMENTED_MSG)

    def rename(self, path, name) -> bool:
        raise NotImplementedError(NOT_IMPLEMENTED_MSG)

    def all_files(self) -> dict:
        raise NotImplementedError(NOT_IMPLEMENTED_MSG)

    def exists(self, name) -> bool:
        raise NotImplementedError(NOT_IMPLEMENTED_MSG)

    def create_folder(self, name) -> bool:
        raise NotImplementedError(NOT_IMPLEMENTED_MSG)

    def delete_folder(self, name) -> bool:
        raise NotImplementedError(NOT_IMPLEMENTED_MSG)

    def delete_file(self, name) -> bool:
        raise NotImplementedError("Not implemented")

    def convert_bytes(self, num) -> str:
        """
        this function will convert bytes to MB.... GB... etc
        """
        step_unit = 1000.0  # 1024 bad the size

        for x in ["bytes", "KB", "MB", "GB", "TB"]:
            if num < step_unit:
                return "%3.1f %s" % (num, x)
            num /= step_unit

    def validate(self, data: dict) -> dict:
        try:
            if data.get("total_size") is None:
                raise ValueError("total_size key is required")

            if data.get("folders") is None:
                raise ValueError("folders key is required")

            if data.get("files") is None:
                raise ValueError("files key is required")

            data["folders"] = self.verify_folder_data(data["folders"])
            data["files"] = self.verify_file_data(data["files"])

            return data

        except Exception as e:
            raise ValueError(e)

    def verify_folder_data(self, folders: list) -> list:
        try:
            filtered_folders = []

            for folder in folders:
                if all(
                    key in folder
                    for key in ["name", "size", "path", "created", "modified", "total_files"]
                ):
                    filtered_folders.append(folder)

            return filtered_folders

        except Exception as e:
            raise ValueError(e)

    def verify_file_data(self, files: list) -> list:
        try:
            filtered_files = []

            for file in files:
                if all(
                    key in file
                    for key in ["name", "size", "path", "created", "modified", "url", "mime"]
                ):
                    filtered_files.append(file)
            return filtered_files

        except Exception as e:
            raise ValueError(e)



"""UserTableSeeder Seeder."""
from masoniteorm.seeds import Seeder
from masonite.facades import Hash

from tests.integrations.app.models.User import User


class UserTableSeeder(Seeder):
    def run(self):
        """Run the database seeds."""
        User.create(
            {
                "name": "Joe",
                "email": "user@example.com",
                "password": Hash.make("secret"),
                "phone": "+123456789",
            }
        )



"""A FileManagerProvider Service Provider."""

from masonite.packages import PackageProvider


class FileManagerProvider(PackageProvider):
    def configure(self):
        """Register objects into the Service Container."""
        (
            self.root("filemanager")
            .name("filemanager")
            .config("config/filemanager.py", publish=True)
            .views("templates", publish=False)
            .assets("resources")
            .routes("routes/route.py")
        )

    def register(self):
        super().register()

    def boot(self):
        """Boots services required by the container."""
        pass
